@@ -1,5 +1,5 @@
 /*  ADCD - A Diminutive CD player for GNU/Linux
-    Copyright (C) 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -150,7 +150,8 @@ void CD::open() throw()
 void CD::pause() throw()
   {
   if( status == playing ) { ioctl( filedes, CDROMPAUSE ); status = paused; }
-  else if( status == paused ) { ioctl( filedes, CDROMRESUME ); status = playing; }
+  else if( status == paused )
+    { ioctl( filedes, CDROMRESUME ); status = playing; read_status(); }
   }
 
 
