@@ -1,5 +1,5 @@
 /*  ADCD - A Diminutive CD player for GNU/Linux
-    Copyright (C) 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,9 +24,10 @@
 
 #include <cstdio>
 #include <cstring>
+#include <string>
 #include <vector>
-#include <getopt.h>
 
+#include "arg_parser.h"
 #include "msf_time.h"
 #include "cd.h"
 #include "player.h"
@@ -34,12 +35,12 @@
 
 namespace {
 
-// Date of this version: 2005-10-19
+// Date of this version: 2006-04-03
 
 const char * invocation_name = 0;
-const char * const program_name    = "ADCD";
-const char * const program_version = "0.6";
-const char * const program_year    = "2005";
+const char * const program_name    = "Adcd";
+const char * const program_version = "0.7";
+const char * const program_year    = "2006";
 
 
 void show_error( const char * msg, const int errcode = 0, const bool help = false ) throw()
@@ -64,10 +65,10 @@ void show_help() throw()
   std::printf( "  -h, --help             display this help and exit\n" );
   std::printf( "  -V, --version          output version information and exit\n" );
   std::printf( "  -i, --info             display info about current status and exit\n" );
-  std::printf( "  -p, --play=<n>         start playing at track <n> and exit\n" );
+  std::printf( "  -p, --play[=<n>]       start playing at track <n> (default 1) and exit\n" );
   std::printf( "  -s, --stop             stop playing and exit\n" );
   std::printf( "  -u, --pause            pause/resume playing and exit\n" );
-  std::printf( "  -v, --volume=[+-]<n>   adjust or set volume and exit\n" );
+  std::printf( "  -v, --volume[=[+-]<n>] show, adjust or set volume and exit\n" );
   std::printf( "\nKeys not shown on screen:\n" );
   std::printf( "  1 to 9             Start playing at track 1 to 9\n" );
   std::printf( "  0                  Start playing at track 10\n" );
@@ -112,53 +113,55 @@ void show_version() throw()
 
 int main( int argc, char * argv[] ) throw()
   {
-  enum Volcommand { vnone, vadjust, vset };
+  enum Volcommand { vnone, vadjust, vset, vshow };
   Volcommand vc = vnone;
   int track = 1, volume = 0;
   bool info = false, pause = false, play = false, stop = false;
-  const char *devname = "/dev/cdrom";
   invocation_name = argv[0];
 
-  // scan arguments
-
-  while( true )			// process options
+  static const Arg_parser::Option options[] =
     {
-    static const struct option long_options[] =
-      {
-      {"help",         no_argument, 0, 'h'},
-      {"info",         no_argument, 0, 'i'},
-      {"pause",        no_argument, 0, 'u'},
-      {"play",   required_argument, 0, 'p'},
-      {"stop",         no_argument, 0, 's'},
-      {"version",      no_argument, 0, 'V'},
-      {"volume", required_argument, 0, 'v'},
-      {0, 0, 0, 0}
-      };
+    { 'h', "help",    Arg_parser::no    },
+    { 'i', "info",    Arg_parser::no    },
+    { 'p', "play",    Arg_parser::maybe },
+    { 's', "stop",    Arg_parser::no    },
+    { 'u', "pause",   Arg_parser::no    },
+    { 'v', "volume",  Arg_parser::maybe },
+    { 'V', "version", Arg_parser::no    },
+    {  0 , 0,         Arg_parser::no    } };
 
-    int c = getopt_long( argc, argv, "Vhip:suv:", long_options, 0 );
-    if( c == -1 ) break;		// all options processed
+  Arg_parser parser( argc, argv, options );
+  if( parser.error().size() )				// bad option
+    { show_error( parser.error().c_str(), 0, true ); return 1; }
 
-    switch( c )
+  int argind;
+  for( argind = 0; argind < parser.arguments(); ++argind )
+    {
+    const int code = parser.code( argind );
+    if( !code ) break;					// no more options
+    const char * arg = parser.argument( argind ).c_str();
+    switch( code )
       {
-      case 'V':	show_version(); return 0;
       case 'h': show_help(); return 0;
       case 'i': info = true; break;
-      case 'p': play = true; track = std::strtol( optarg, 0, 0 ); break;
+      case 'p': play = true; if( arg[0] ) track = std::strtol( arg, 0, 0 ); break;
       case 's': stop = true; break;
       case 'u': pause = true; break;
-      case 'v': if( isdigit( optarg[0] ) ) vc = vset;
-                else if( ( optarg[0] == '+' || optarg[0] == '-' ) && isdigit( optarg[1] ) )
+      case 'v': if( !arg[0] ) vc = vshow;
+                else if( isdigit( arg[0] ) ) vc = vset;
+                else if( ( arg[0] == '+' || arg[0] == '-' ) && isdigit( arg[1] ) )
                   vc = vadjust;
                 else { show_error( "bad volume value" ); return 1; }
-                volume = std::strtol( optarg, 0, 0 );
+                volume = std::strtol( arg, 0, 0 );
                 break;
-      case '?': show_error( 0, 0, true ); return 1;	// bad option
-      default : show_error( argv[optind], 0, true ); return 1;
+      case 'V':	show_version(); return 0;
+      default : show_error( "internal error: uncaught option" ); return 3;
       }
     } // end process options
 
-  if( optind < argc ) devname = argv[optind++];
-  if( optind < argc )
+  const char *devname = "/dev/cdrom";
+  if( argind < parser.arguments() ) devname = parser.argument( argind++ ).c_str();
+  if( argind < parser.arguments() )
     { show_error( "too many devices", 0, true ); return 1; }
 
   // end scan arguments
@@ -169,9 +172,13 @@ int main( int argc, char * argv[] ) throw()
   if( vc != vnone )
     {
     interactive = false;
-    if( vc == vadjust ) cd.volume( cd.volume() + volume );
-    else if( vc == vset ) cd.volume( volume );
-    std::printf( "New volume = %d\n", cd.volume() );
+    if( vc == vshow ) std::printf( "Current volume = %d\n", cd.volume() );
+    else
+      {
+      if( vc == vadjust ) cd.volume( cd.volume() + volume );
+      else if( vc == vset ) cd.volume( volume );
+      std::printf( "New volume = %d\n", cd.volume() );
+      }
     }
 
   if( stop ) { cd.stop(); interactive = false; }
